import mariadb
import csv
import os

def initialize_conn():
        #Establish the connection
        try:
               conn = mariadb.connect(
                       user=os.getenv('DATABASE_USERNAME'),
                       password=os.getenv('DATABASE_PASSWORD'),
                       host=os.getenv('DATABASE_HOST'),
                       port=3306,
                       database=os.getenv('DATABASE_NAME')
               )
        except mariadb.Error as e:
                raise Exception(e)
        
        cursor = conn.cursor()
        return conn, cursor
        
       
    
def close_conn(conn):
    conn.close()
    
def insert_data(conn, cursor, data):
    #expecting data will be a JSON object
    table_name = 'PollutionData'
    
    #GPS module outputs have N/S/E/W instead of +/- lat and long
    row = modify_coordinates(data)
    
    date=row['date_measured']
    time=row['time_measured']
    co2=row['co2']
    temp=row['temperature']
    humidity=row['humidity']
    pm25=row['pm25']
    lat = row['latitude']
    long=row['longitude']

    

    try:
        cursor.execute("INSERT INTO PollutionData (DateMeasured, TimeMeasured, Latitude, Longitude, CO2, Temperature, Humidity, PM25) VALUES (?, ?, ?, ?, ?, ?, ?, ?)", (date, time, lat, long, co2, temp, humidity, pm25))
    except mariadb.Error as e:
        print(f"Error: {e}")
    
    conn.commit()

    #Also save the data into a csv file
    csv_file_path = 'static/output_data.csv'
    file_exists = os.path.exists(csv_file_path)
    with open(csv_file_path, mode="a", newline='') as csv_file:
        fieldnames=['date_measured','time_measured','latitude','longitude','co2','temperature','humidity','pm25']
        writer = csv.DictWriter(csv_file, fieldnames=fieldnames)

        #header
        if not file_exists:
                writer.writeheader()

        #write data
        writer.writerow(row)

def get_variation(cursor):
       query = "SELECT TimeMeasured, CO2, PM25, Temperature, Humidity FROM PollutionData WHERE DateMeasured IN (SELECT max(DateMeasured) FROM PollutionData)"
       time = []
       co2 = []
       temp = []
       humidity = []
       pm25 = []

       try:
             cursor.execute(query)
             for item in cursor:
                    time.append(item[0])
                    co2.append(item[1])
                    pm25.append(item[2])
                    temp.append(item[3])
                    humidity.append(item[4])
             return time, co2, pm25, temp, humidity
       except mariadb.Error as e:
             print(f"Error: {e}")
       
def get_latest(cursor):
       query = "SELECT CO2, PM25, Temperature, Humidity, Latitude, Longitude FROM PollutionData WHERE DateMeasured IN (SELECT max(DateMeasured) FROM PollutionData) AND TimeMeasured IN (SELECT MAX(TimeMeasured) FROM PollutionData WHERE DateMeasured IN (SELECT MAX(DateMeasured) FROM PollutionData))"
       
       co2 = "0"
       temp = "0"
       humidity = "0"
       pm25 = "0.0"
       lat = "0.0"
       longi = "0.0"
       
       try:
              cursor.execute(query)
              for item in cursor:
                    co2 = item[0]
                    pm25 = item[1]
                    temp = item[2]
                    humidity = item[3]
                    lat = item[4]
                    longi = item[5]
              return co2, pm25, temp, humidity, lat, longi
       except mariadb.Error as e:
              print(f"Error: {e}")

def get_heatmap_points(cursor):
       query = "SELECT DISTINCT(CONCAT(Latitude, ' ', Longitude)) AS cc, CO2 FROM PollutionData pd WHERE DateMeasured IN (SELECT max(DateMeasured) FROM PollutionData) AND pd.Latitude NOT IN ('-0.000000','0.0000000') GROUP BY cc"

       lat_points = []
       long_points = []
       co2_conc = []

       try:
             cursor.execute(query)
             for item in cursor:
                    lat = item[0].split(" ")[0]
                    long = item[0].split(" ")[1]
                    lat_points.append(float(lat))
                    long_points.append(float(long))
                    co2_conc.append(item[1])
             return lat_points, long_points, co2_conc
       except mariadb.Error as e:
             print(f"Error: {e}")

def modify_coordinates(row_data):
        #N/E are +ve coordinates, S/W are -ve coordinates
        lat_string=row_data['latitude']
        long_string=row_data['longitude']
        #if lat_string != "":
        #        lat=lat_string[:-1] if lat_string[-1]=="N" else "-"+lat_string[:-1]
        #else:
        #        lat = lat_string
        #if long_string != "":
        #        long=long_string[:-1] if long_string[-1]=="E" else "-"+long_string[:-1]
        #else:
        #        long = long_string
        row_data['latitude']=lat_string
        row_data['longitude']=long_string
        return row_data
        
